/* gindexedlist - an indexed list, wanna be part of the glib.
 * Copyright (C) 2004  Kristian Benoit
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __G_INDEXED_LIST_H__
#define __G_INDEXED_LIST_H__

typedef struct g_indexed_list_St GIndexedList;
typedef struct g_indexed_node		GIndexedNode;
typedef gint	(*GICompareFunc)		( gconstpointer key_a,
												  gconstpointer value_a,
												  gconstpointer key_b,
												  gconstpointer value_b );
typedef gint	(*GICompareDataFunc)	( gconstpointer key_a,
												  gconstpointer value_a,
												  gconstpointer key_b,
												  gconstpointer value_b,
												  gpointer		 user_data );

/* Constructors. */
GIndexedList *g_indexed_list_new 		( GHashFunc 		hash_func,
													  GEqualFunc 		key_equal_func );
GIndexedList *g_indexed_list_copy 	   ( GIndexedList   *indexed_list );
GIndexedList *g_indexed_list_new_full 	( GHashFunc 		hash_func,
													  GEqualFunc 		key_equal_func,
													  GDestroyNotify 	key_destroy_func,
													  GDestroyNotify 	value_destroy_func );
GIndexedList *g_indexed_list_destroy 	( GIndexedList   *indexed_list );

/* adding nodes */
void g_indexed_list_append				( GIndexedList	  *indexed_list,
												  gpointer			key,
												  gpointer			value );

void g_indexed_list_prepend			( GIndexedList	  *indexed_list,
												  gpointer 			key,
												  gpointer 			value );

void g_indexed_list_insert 			( GIndexedList	  *indexed_list,
												  gpointer			key,
												  gpointer			value,
												  guint				position );

void g_indexed_list_insert_before 	( GIndexedList	  *indexed_list,
												  gpointer			sibling_key,
												  gpointer			key,
												  gpointer			value );

void g_indexed_list_insert_sorted 	( GIndexedList	  *indexed_list,
												  gpointer			key,
												  gpointer			value,
												  GCompareFunc 	func );

void g_indexed_list_replace 			( GIndexedList	  *indexed_list,
												  gpointer			key,
												  gpointer			value );

/* move multiple nodes with check for valid parameters. O(n)*/
void g_indexed_list_move_first 		( GIndexedList   *indexed_list,
												  gpointer			start_key,
												  gpointer			end_key );

void g_indexed_list_move_last 		( GIndexedList	  *indexed_list,
												  gpointer			start_key,
												  gpointer	 		end_key );

void g_indexed_list_move_forward 	( GIndexedList	  *indexed_list,
												  gpointer			start_key,
												  gpointer			end_key,
												  guint				position );

void g_indexed_list_move_backward 	( GIndexedList	  *indexed_list,
												  gpointer			start_key,
												  gpointer			end_key,
												  guint				position );

void g_indexed_list_move_before 		( GIndexedList	  *indexed_list,
												  gpointer			start_key,
												  gpointer			end_key,
												  gpointer			sibling_key );

void g_indexed_list_move_after 		( GIndexedList	  *indexed_list,
												  gpointer			start_key,
												  gpointer			end_key,
												  gpointer			sibling_key );

/* move multiple nodes without check for valid parameters. O(1) */
void g_indexed_list_move_first_nocheck			( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );

void g_indexed_list_move_last_nocheck			( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );

void g_indexed_list_move_forward_nocheck		( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key,
															  guint				position );

void g_indexed_list_move_backward_nocheck		( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key,
															  guint				position );

void g_indexed_list_move_before_nocheck		( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key,
															  gpointer			sibling_key );

void g_indexed_list_move_after_nocheck			( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key,
															  gpointer			sibling_key );

/* move single node */
#define g_indexed_list_move1_first(indexed_list, key) \
	g_indexed_list_move_first_nocheck (indexed_list, key, key)

#define g_indexed_list_move1_last(indexed_list, key) \
	g_indexed_list_move_last_nocheck (indexed_list, key, key)

#define g_indexed_list_move1_forward(indexed_list, key, position) \
	g_indexed_list_move_forward_nocheck (indexed_list, key, key, position)

#define g_indexed_list_move1_backward(indexed_list, key, position) \
	g_indexed_list_move_backward_nocheck (indexed_list, key, key, position)

#define g_indexed_list_move1_before(indexed_list, key, sibling_key) \
	g_indexed_list_move_before_nocheck (indexed_list, key, key, sibling_key)

#define g_indexed_list_move1_after(indexed_list, key, sibling_key) \
	g_indexed_list_move_after_nocheck (indexed_list, key, key, sibling_key)


/* removing nodes */
gboolean g_indexed_list_remove 					( GIndexedList	  *indexed_list,
															  gpointer			key );
gboolean g_indexed_list_steal 					( GIndexedList	  *indexed_list,
															  gpointer			key );
guint		g_indexed_list_foreach_remove			( GIndexedList	  *indexed_list,
															  GHRFunc			func,
															  gpointer			user_data );
guint		g_indexed_list_foreach_steal			( GIndexedList	  *indexed_list,
															  GHRFunc			func,
															  gpointer			user_data );
guint		g_indexed_list_crop						( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );
guint		g_indexed_list_remove_sub				( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );
guint		g_indexed_list_crop_nocheck			( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );
guint		g_indexed_list_remove_sub_nocheck 	( GIndexedList	  *indexed_list,
															  gpointer			start_key,
															  gpointer			end_key );
void		g_indexed_list_clear						( GIndexedList	  *indexed_list );

/* length info */
guint		g_indexed_list_length					( GIndexedList		*indexed_list );
guint		g_indexed_list_length_before			( GIndexedList		*indexed_list,
															  gpointer			 key );
guint		g_indexed_list_length_after			( GIndexedList		*indexed_list,
															  gpointer			 key );

/* manipulation */
void		g_indexed_list_reverse					( GIndexedList		  *indexed_list );
void		g_indexed_list_sort 						( GIndexedList		  *indexed_list,
															  GICompareFunc		compare_func );
void		g_indexed_list_sort_with_data 		( GIndexedList		  *indexed_list,
															  GICompareDataFunc	compare_func,
															  gpointer				user_data );
void		g_indexed_list_shuffle 					( GIndexedList		  *indexed_list );
void		g_indexed_list_foreach					( GIndexedList		  *indexed_list,
															  GHFunc					func,
															  gpointer				user_data );


/* accessor */
gpointer		g_indexed_list_lookup				( GIndexedList	  *indexed_list,
															  gpointer			key );
gboolean		g_indexed_list_lookup_extended 	( GIndexedList	  *indexed_list,
															  gconstpointer	lookup_key,
															  gpointer		  *orig_key,
															  gpointer		  *value );

gpointer		g_indexed_list_first 				( GIndexedList	  *indexed_list );

gpointer		g_indexed_list_last 					( GIndexedList	  *indexed_list );

gpointer		g_indexed_list_previous 			( GIndexedList	  *indexed_list,
															  gpointer			key );

gpointer		g_indexed_list_next 					( GIndexedList	  *indexed_list,
															  gpointer			key );

gpointer		g_indexed_list_nth 					( GIndexedList	  *indexed_list,
															  gpointer			key,
															  guint				n );

gpointer		g_indexed_list_nth_prev 			( GIndexedList	  *indexed_list,
															  gpointer			key,
															  guint				n );

gpointer		g_indexed_list_find 					( GIndexedList	  *indexed_list,
															  GHRFunc			predicate,
															  gpointer			user_data );


#endif
